package be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.game;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.listener.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.geom.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.gui.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.*;

import java.awt.geom.Point2D;
import java.awt.geom.Ellipse2D;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.awt.Image;
import java.awt.Graphics2D;
import javax.swing.JPanel;
import java.util.Random;
public class Asteroid extends Moving implements IDrawable
{
    private double radius;
    private double lastCollWithAsteroid = 0; //mmorise le temps de la derniere collision avec un asteroides (vite les collisions en cascade quand il y a plusieurs asteroides l'un  cot de l'autre
    private final int IWHIDTH = 100, IHEIGTH = 100; // taille de l'image d'asterodes
    
    
    /**
    * Construits un astrode.
    * @param pPos position de l'astrode.
    * @param vDirect direction de l'astrode.
    * @param speed vitesse de l'astrode.
    * @param radius rayon de l'astrode.
    */
    public Asteroid(Point2D.Double pPos, Point2D.Double vDirect, double speed, double radius)
    {
        
        super(pPos, vDirect, speed % 100);
        this.radius = radius;
    }
    
    /**
    * Construits un astrode avec un radius par defaut (de valeur 30).
    * @param pPos position de l'astrode.
    * @param vDirect direction de l'astrode.
    * @param speed vitesse de l'astrode.
    */
    public Asteroid(Point2D.Double pPos, Point2D.Double vDirect, double speed)
    {
        super(pPos, vDirect, speed);
        this.radius = 30;
    }
    
    
    /**
    * Crer un nouvel astrode de faon alatoire.
    * Le vecteur vitesse est gnr alatoirement avec une norme comprise entre 0 et 30.
    * @return Un nouvelle asteroide.
    */
    public static Asteroid randomAsteroid()
    {
        // les asteroides sont ajout  une distance longeur de diagonal de la fenetre du vaisseau.
        Point2D.Double position = Moving.generatePosition();
        // on construit un vecteur direction qui se dirige vers le vaisseau avec un angle choisi en fonction du niveau.
        
        Point2D.Double direction = generateDirection(position);
        return new Asteroid( position , direction, (int) (100 * Math.random() + 80), (Math.random() * 30 + 10) );
    }
    

    
    /**
    * Converti un astrode en une llipse.
    * @return une llipse.
    */
    public Ellipse2D.Double asteroidToEllipse()
    {
        return asteroidToEllipse(getPos(), getRadius());
    }    
    
    /**
    * Converti un astrode en une ellipse.
    * @param p La position de l'astrode.
    * @param r le radius de l'astrode.
    * @return une ellipse.
    */
    public static Ellipse2D.Double asteroidToEllipse(Point2D.Double p, double r)
    {
        Point2D.Double pUpLeftCorner = new Point2D.Double(p.getX() - r, p.getY() - r);
        return new Ellipse2D.Double(pUpLeftCorner.getX(), pUpLeftCorner.getY(), r * 2, r * 2);
    }


    /**
    * @return la taille du radius de l'astrode.
    */
    public double getRadius()
    {
        return radius;
    }

    /**
    * Change le rayon de l'astrode.
    * @param r un nouveau rayon.
    */
    public void setRadius(double r)
    {
        radius = r;
    }


    /**
    * Compare 2 astrodes.
    * @return true si les 2 astrodes sont identiques,false sinon.
    */
    public boolean equals(Object other)
    {
        Asteroid otherAsteroide = (Asteroid) other;
        return (getPos().equals(otherAsteroide.getPos())) && (getRadius() == otherAsteroide.getRadius()) && (getSpeed() == otherAsteroide.getSpeed()) && (getDirect().equals(otherAsteroide.getDirect()));
    }
    
    /**
    * @return une position pour l'astrode de sorte  simuler le dbordement.
    */
    public Point2D.Double pCloseBorder()
    {
        return Geom.pCloseBorder(getPos(), getRadius(), Config.getFrameX(), Config.getFrameY());
    }
    
    /**
     * Mthode pour dessiner un astrode.
     */
    public void draw(Graphics g)
    {
            // affiche l'image
            Graphics2D g2 = (Graphics2D) g;
            AffineTransform at = new AffineTransform();
            //Calcul pour avoir le coin superieur gauche de l'image
            Point2D.Double v = Geom.normalize(Geom.inverseVector(getOrien()),getRadius());
            Point2D.Double pCircle = Geom.translate(new Point2D.Double(getPos().getX(), getPos().getY()), Geom.normalize(getOrien(),getRadius()));
            Point2D.Double pCorner = Geom.translate(pCircle,Geom.rotate(v,90)) ;
            
            // place l'image
            at.translate(pCorner.getX() * Config.getFrameRate(), pCorner.getY() * Config.getFrameRate());
            at.rotate(Math.toRadians(Geom.angle(getOrien())+90)); // tourne l'image
            
            //on calcule le ratio pour diminuer la taille de l'image
            double ratio = ((getRadius() * 2) / IWHIDTH)  * Config.getFrameRate();
            at.scale(ratio, ratio); // change la taille de l'image
            g2.drawImage(Engine.images.asteroid, at, null); 
            // rotation de l'asterode
            setOrien(Geom.rotate(getOrien(),Config.getTurnFactor()/5D));
    }
 
    
    /**
    * @return si l'astroide peut-tre dessin dans la zone centrale du jeu.
    */
    public boolean isIDrawable()
    {
        return (( Geom.distance(getPos(), Config.getPCenter()) - getRadius() ) <= Config.getDistancePrintMobile() );
    }
    
    /**
    * Le choix de regarder aussi la distance vient du fait que le bonus division entraine un nombre trop important d'asteroides
    * @return si l'astrode est asses grand et asses prs pour tre divis
    */
    public boolean isDivisible()
    {
        return ((getRadius() / Config.getCurrentAsteroidRadiusDivFactor()) >= Config.getCurrentAsteroidRadiusMin()) && isIDrawable();
    }
    
    
    
    /**
    * @return si l'asteroide  deja tait en collision avec un autre asteroide  ce tic
    */
    public boolean canBeCollWithAsteroid()
    {
        return (PlayTime.getTime() > lastCollWithAsteroid) && isIDrawable();
    }
    
    /**
    * Modifie la variable lastCollWithAsteroid pour indiquer le temps de la derniere collision avec un astrode.
    */
    public void setCollWithAsteroid()
    {
        lastCollWithAsteroid = PlayTime.getTime();
    }
    
}