package be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.game;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.listener.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.geom.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.gui.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.*;

import java.awt.geom.Point2D;
import java.awt.geom.Ellipse2D;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.awt.Image;
import java.awt.Graphics2D;
import javax.swing.JPanel;
import java.util.Random;
public class Astronaut extends Moving implements IDrawable
{
    private double timeRescue; // temps de sauvetage
    private final int IWHIDTH = 80, IHEIGTH = 80; // taille de l'image d'asterodes
    private static double radius = Config.getSpaceManRadius();
    private double tCollWave; // temps de collision avec l'onde du radar
    private Point2D.Double pCollWave; 

    /**
    * Construits un astronaute
    */
    public Astronaut()
    {
        // on gnre une position allatoire.
        // direction allatoire
        super(Moving.generatePosition(), Geom.vectorRandom(), 10); //vitesse de 10
        timeRescue = Config.getSpaceManTimeCapture(); 
        tCollWave = 0;
        pCollWave = null;
    }
    
    /**
    *   Mets  jours la position sans modulo
    */
    public void refreshPosition()
    {
        refreshPosition(false);
    }  
    
    /**
    * @return si le missile peut-tre dessin dans la zone centrale du jeu.
    */
    public boolean isIDrawable()
    {
        return (( Geom.distance(getPos(), Config.getPCenter()) - getRadius() ) <= Config.getDistancePrintMobile() );
    }
    
    /**
    *   @return Le rayon des astronautes
    */
    public double getRadius()
    {
        return radius;
    }
    
    /**
    *  @return si l'on peut sauver l'astronaute.
    */
    public boolean isSaved()
    {
        return timeRescue <= 0;
    }
    
    /**
    * Diminue le temps de secours encore  faire pour capturer l'astronaute.
    * @param t un temps en milliseconde
    */
    public void decreaseTimeRescue(double t)
    {
        timeRescue -= t;
    }
    
    /**
    * @return l'angle de sauvetage.
    */
    public double getAngleSaved()
    {
        if (timeRescue > 0)
        {
            return 360 - (timeRescue / Config.getSpaceManTimeCapture()) * 360;
        }
        else 
            return 360;
    }
    
    
    
    /**
     * Mthode pour dessiner un astronaute.
     */
    public void draw(Graphics g)
    {

        /* 
        // dssine un ovale rouge
        g.setColor(Color.red);
        g.fillOval((int) (getPos().getX() - getRadius()), (int)  (getPos().getY() - getRadius()), (int) (getRadius() * 2), (int) (getRadius() * 2) );  
        */
        
        /* On va dssiner la barre de sauvetage */
        // le rayon du cercle interne doit tre de 22px
        
        //On va calculer le rayon du cercle interne
        double rCircleInt = getRadius() * 0.9;
        
        // cercle rouge
        g.setColor(Color.red);
        g.fillOval((int) ( (getPos().getX() - rCircleInt) * Config.getFrameRate()), (int)  ((getPos().getY() - rCircleInt) * Config.getFrameRate()), (int) (rCircleInt * 2 * Config.getFrameRate()), (int) (rCircleInt * 2 * Config.getFrameRate()) );
        
        // arc de cercle vert > grandit plus le temps restant de sauvetage est rduit
        // on calcule l'angle avec le quel l'astronaute est tourn
        double angle = - Geom.angle(getOrien()) ;
        g.setColor(Color.green);
        g.fillArc((int) ( (getPos().getX() - rCircleInt) * Config.getFrameRate()), (int)  ((getPos().getY() - rCircleInt) * Config.getFrameRate()), (int) (rCircleInt * 2 * Config.getFrameRate()), (int) (rCircleInt * 2 * Config.getFrameRate()), (int) (angle - (getAngleSaved() / 2 )), (int) getAngleSaved() );

        
        /* Maintenant on s'occupe de l'image */
        
        Graphics2D g2 = (Graphics2D) g;
        AffineTransform at = new AffineTransform();
        //On donne  l'astronaute une direction orient vers le vaiseau
        setOrien(Geom.normalize(Geom.vector(getPos(), Config.getPCenter()), getRadius()));
        // on inverse le vecteur orientation qu'on normalize avec la taille du rayon
        Point2D.Double v = Geom.inverseVector(getOrien());  
        //on translate la position avec le vecteur orient (on obtient un point sur le cercle)
        Point2D.Double pCircle = Geom.translate(new Point2D.Double(getPos().getX(), getPos().getY()), getOrien());
        // on ajoute le point du cercle avec le vecteur orient qui a subi une inversion et une rotation de 90 degres        
        Point2D.Double pCorner = Geom.translate(pCircle,Geom.rotate(v,90)) ; 
        
        // place l'image
        at.translate(pCorner.getX() * Config.getFrameRate(), pCorner.getY() * Config.getFrameRate());
        // tourne l'image
        at.rotate(Math.toRadians(Geom.angle(getOrien())+90));
        //on calcule le ratio pour diminuer la taille de l'image
        double rate = ((getRadius() * 2) / IWHIDTH)  * Config.getFrameRate();
        at.scale(rate, rate); // change la taille de l'image
        g2.drawImage(Engine.images.astronaut, at, null);
        
    }
    
    /* RADAR */
    
    
    /**
    * Permet de mettre  jour le temps de collision des astronautes avec l'onde du radar
    */
    public void setCollWave()
    {
        tCollWave = PlayTime.getTime();
        pCollWave = getPos();
    }
    
    /**
    * @return le point de collision avec l'onde du radar.
    */
    public Point2D.Double getPCollWave()
    {
        return pCollWave;
    }    
    
    /**
    * @return le moment o l'onde heurte le centre de gravit de l'astronaute
    */
    public double getTCollWave()
    {
        return tCollWave;
    }
    
}