package be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.game;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.listener.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.geom.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.gui.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.*;

import java.util.ArrayList;
import java.awt.Polygon;
import java.awt.geom.Point2D;
import java.awt.Graphics;

import java.awt.Font;
import java.awt.Color;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.Graphics2D;
public class Radar implements IDrawable
{

    /* on en mmorise 2 car il faut l'ancien et le courant pour l'affichage) */
    private double tImpulsionBonusEnd; // temps jusqu'au quel on a un bonus radar
    private double tImpulsionBreackEnd; //fin du bonus "iem".
    private static final double PERIOD = 3000; // temps pour faire un tour
    private ArrayList<Astronaut> arrayAstronaut;
    private double angleWave; //angle courant de l'onde
    private static final double ANGLEWAVE = 17D; //l'amplitude de l'onde est de 17
    /*on calcule la longeur de l'onde */
    private RadarWave radarWave; 
        
   
    public Radar( ArrayList<Astronaut> list)
    {
        arrayAstronaut = list;
        tImpulsionBonusEnd = 0;
        tImpulsionBreackEnd = 0;
        angleWave = 0;
        refresh(); //lance le radar
        radarWave = new RadarWave();
    }
    
    
    /**
     * raffraichit les impulsions;
     */
    public void refresh()
    {        
        // on va parcourir les astronautes pour mettres  jours leurs point de collison avec le radar
        
        //Si on n'est pas sous l'influeance d'un bonus IEM on fait l'affichage
        if (!isBreack())
        {
            for(Astronaut astro : arrayAstronaut)
            {
                //on va regarde si l'astronaute est dans la zone de dtection du radar
                double angle = Geom.angle(Geom.vector(Config.getPCenter(), astro.getPos()));
                if (angle >= angleWave - ANGLEWAVE && angle <= angleWave)
                {
                    astro.setCollWave();
                }
            }
            angleWave = (angleWave + getSpeed()) % 360;
        }
    }
    
        
    /**
    * @return si on est sous l'influence d'un bonus radar.
    */
    public boolean isUnderInfluenceBonus()
    {
        return PlayTime.getTime() <= tImpulsionBonusEnd;
    }
    
    /**
    * @return si le radar est en pause (cass par le malus IEM).
    */
    public boolean isBreack()
    {
        return PlayTime.getTime() <= tImpulsionBreackEnd;
    }
    
    /**
    * @param t un double
    */
    public void setTImpulsionBreackEnd(double t)
    {
        tImpulsionBreackEnd = PlayTime.getTime() + t;
    }
    
    /**
    * @param t un double
    */
    public void setTImpulstionBonusEnd(double t)
    {
        tImpulsionBonusEnd = PlayTime.getTime() + t;
    }
    
    /**
     * Donne la vitesse de l'impusion (en tenant compte qu'on est sous l'effet d'un bonus ou non).
     */
    public double getSpeed()
    {
        double speed = (360D / PERIOD) * Config.getRefresh();
        if (isUnderInfluenceBonus())
            return speed * 2D;
        return speed;
    }
    
    

    
    /**
    * Affiche les astronautes autour de la zone de jeu
    */
    public void draw(Graphics g)
    {
        for (Astronaut astro : arrayAstronaut)
        {
            // Si l'astronaute a dj tait dtecter par le radar et que il est pas trop prs du vaisseau
            if(astro.getPCollWave() != null && Geom.distance(Config.getPCenter(), astro.getPCollWave()) > Config.getDistancePrintMobile() )
            {
                /* Calcule de l'opacit de l'image */
                //On prend le max entre 0 et la difference car si mal IEM le temps sera ngatif
                double opacity = 100 - (Math.max((PlayTime.getTime() - astro.getTCollWave()), 0) / PERIOD)  * 100 ;
                //si l'oppacyt est > 0 on affiche les infos
                if (opacity > 0)
                {
                    //dessine l'astronaute
                    Graphics2D g2 = (Graphics2D) g;
                    AffineTransform at = new AffineTransform();
                    
                    /* Positionnement de l'image */
                    
                    //Vecteur qui va du centre de la fenetre vers l'astronaute
                    Point2D.Double v = Geom.normalize(Geom.vector(Config.getPCenter(), astro.getPCollWave()), Config.getDistanceDrawMobile());
                    //point du centre de l'astronaute
                    Point2D.Double pCenter = Geom.add(Config.getPCenter(), v);
                    Point2D.Double pCorner = Geom.translate(pCenter, new Point2D.Double(-20, -20));
                    at.translate(pCorner.getX() * Config.getFrameRate(), pCorner.getY() * Config.getFrameRate());
                    
                    at.scale(Config.getFrameRate(), Config.getFrameRate());
                    g2.drawImage(Engine.images.getImageRadarAstro(opacity), at, null);
                    
                    /* On va afficher la distance  la quel est l'astronaute */
                    int fontSize = (int) ( 20 * Config.getFrameRate());
                    g.setFont(new Font(GuiEvent.FONT, Font.PLAIN, fontSize));
                    g.setColor(GuiEvent.GREEN); // vert
                    //texte
                    String output = "D:" + (int) Geom.distance(Config.getPCenter(), astro.getPCollWave());
                    // On va calculer la position du texte en fct de la position de l'astronaute
                    
                    // On regarde dans quel quadrant on est
                    double angle = Geom.angle(v);
                    if (angle <= 90) //4eme quadrant
                    {
                        // on deplace vers le bord superieur droit
                        pCorner = Geom.translate(pCenter, new Point2D.Double(10, -10));
                    }                    
                    else if (angle <= 180) //3eme quadrant
                    {
                        // on deplace vers le bord superieur gauche
                        pCorner = Geom.translate(pCenter, new Point2D.Double(-70, -10));
                    }                    
                    else if (angle <= 270) //2eme quadrant
                    {
                        // on deplace vers le bord inferieur gauche
                        pCorner = Geom.translate(pCenter, new Point2D.Double(-70, 10));
                    }                    
                    else //1er quadrant
                    {
                        // on deplace vers le bord infrieur droit
                        pCorner = Geom.translate(pCenter, new Point2D.Double(10, 10));
                    }
                    g.drawString(output, (int) (pCorner.getX() * Config.getFrameRate()), (int) (pCorner.getY()  * Config.getFrameRate())); 
                }
            }
        }
    
    }
    
    
    public class RadarWave implements IDrawable
    {
        public void draw(Graphics g)
        {
            //si le radar n'est pas cass
            if (!isBreack())
            {
                //dessine l'onde radar
                Graphics2D g2 = (Graphics2D) g;
                AffineTransform at = new AffineTransform();
                //334 = hauteur de l'onde
                Point2D.Double pCorner = Geom.translate(Config.getPCenter(), Geom.rotate(new Point2D.Double(0, -334) , angleWave));
                // Point2D.Double pCorner = Config.getPCenter();
                at.translate(pCorner.getX() * Config.getFrameRate(), pCorner.getY() * Config.getFrameRate());
                at.rotate(Math.toRadians(angleWave));
                at.scale(Config.getFrameRate(), Config.getFrameRate());
                g2.drawImage(Engine.images.radarWave, at, null);
            }
        }
    }
    
    /**
     * @return un objet RadarWave qui dssine l'onde radar.
     */
    public RadarWave getRadarWave()
    {
        return radarWave;
    }

}