package be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.game;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.listener.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.geom.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.gui.*;
import be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.*;

import java.awt.Polygon;
import java.awt.geom.Point2D;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.Graphics2D;
import javax.swing.JPanel;
public class SpaceShip extends Moving implements IDrawable
{
    private int accel = 0;
    private double radius; // permet d'avoir un rayon pour le vaisseau, utile pour approximer les collisions.
    private static final double SPEEDMAX = 250;
    private static final double ACCELMAX = 150;
    
    private Point2D.Double vTranslate = new Point2D.Double(); //vecteur de translation du dplacement du vaiseau
    private double lastUpdateTranslate; //temps de la derniere mise a jours du vecteur translation
    private Point2D.Double[] peaks; //on memorise les 3 coins du vaisseau pour ne pas les recalculer pour chaque collision
    
    /* Missile launcher */
    private double angleML; // angle entre le vecteur orient et le vecteur orient du lance missile
    private Point2D.Double posML; // Position de la base du lance missile
    private final double ANGLEMLMAX = 42; //angle de rotation maximum du lance missile par apport au vaisseau
    private double lastMLFire; //temps depuis le dernier tir.
    private static final double MAXTEMPML = 100; //temprature max du canon
    private double tempML; // temprature du canon
    private final double TEMPSLOOSERATE = 0.02; // fixe la perte de temprature par seconde  20.
    private boolean overheatingML; // indique si il y a eu surchauffe
    private final double ENDOVERHEATINGML = 45; // temperature de fin de suchauffe
    private boolean blockingML; //indique si le joeur a bloqu le lance missile vers la pointe du vaisseau    
    private double timeBlockingML; //on enregistre l'instant o il ya changement d'tat pour ne pas changer d'tat trop vite    
    private double timeBonusTemp; //temps de fin du bonus
    private double timeMalusTemp; //temps de fin du malus
    
    /* Variable pour la GUI */
    private double angle; // angle de construction du vaisseau.
    private double gToTarget; //distance entre G et un point du vaisseau (autre que la pointe avant).
    private final double IHEIGHT = 62, IWIDTH = 100; // base et hauteur de l'image du vaiseau
    private final double HEIGHT = IWIDTH * 0.92, BASE = IHEIGHT * 0.74, BACK = IWIDTH * 0.192; // base, hauteur du vaisseau et recule du centre de gravit du lance missile par apport au centre de gravit
    
    private double sizeML = IWIDTH * (1/3D); //longeur du canon (utilis pour la cration des missiles).


    /**
     * Change la position du vaisseau.
     * Et contruit le vaisseau
     */
    public SpaceShip()
    {
        super(Config.getPCenter());
        construcShip(HEIGHT, BASE); // on fourni la taille du futur vaisseau
        refreshPeaks(); // on gnre les 3 coins
        
        lastMLFire = - Config.getMissileTimeInterFire(); //permet de pouvoir tirer directement
        blockingML = false;
        overheatingML = false;
        tempML = 0;
        timeBlockingML = 0;
        timeBonusTemp = 0;
        timeMalusTemp = 0;
    }

    
    /**
    * Mets  jours le vecteur translation avec un vecteur d'entre
    * Vrifie si on est au bon tic pour rinitialiser ou non le vecteur
    * @param vector un vecteur  ajouter a la translation
    */
    public void setTranslate(Point2D.Double vector)
    {
        if (lastUpdateTranslate  == getLifeTime())
        {
            vTranslate = Geom.add(vTranslate, vector);
        }
        else
        {
            vTranslate = vector;
        }
        
        lastUpdateTranslate = getLifeTime();
    }
    

    
    /**
     * Permet de donner les paramtres pour la construction du vaisseau
     * @param h la hauteur du vaisseau (en pixel)
     * @param b la base du vaisseau (largeur de son arrire) (en pixel)
     */
    public void construcShip(double h, double b)
    {
        // On cherche l'angle qui permettra de trouver les 3 points du vaiseau  partir d'une rotation du vecteur orientation
        angle = 180 - Math.toDegrees(Math.atan( (3 * b) / (2 * h))); // (b/2)/(2h/3)
        // distance entre un point de la base du vaisseau et le centre de graviter
        gToTarget = Math.sqrt(0.25 * b * b + (1D/9) * h * h);
        
        setOrien( Geom.normalize(getOrien(), (h * (2D/3))) );
        setDirect( getOrien() );
        radius = Math.max( (h * (2D/3)), gToTarget);
        sizeML = IWIDTH/3D;
        posML = Geom.add(getPos(), Geom.normalize(getOrien(), - BACK)); 
    }
    
    /**
    * @param pointer Un point vers le quel le lance missile va s'orienter
    */
    public void rotateML(Point2D.Double pointer)
    {
        //on calcule le vecteur entre la base du canon et le pointer
        Point2D.Double vectorMLP = Geom.vector(posML, pointer); 
        
        //angle entre le vecteur orientation et le vecteur de la base du canon > le pointeur de souris
        double angleMLP = Geom.angle(getOrien(), vectorMLP); 
        //angle compris entre getTurnFactor() et - getTurnFactor()
        if (Math.abs(angleMLP - angleML) < Config.getTurnFactor())
        {
            angleML = angleMLP;
        } 
        //angle compris entre getTurnFactor() et 180
        else if ((angleMLP - angleML) >= Config.getTurnFactor() && (angleMLP - angleML) <= 180)
        {
            angleML += Config.getTurnFactor();
        }     
        // angle compris entre -getTurnFactor() et -180
        else
        {
            angleML -= Config.getTurnFactor();
        }
        
        // On fait le modulo ANGLEMLMAX
        if (angleML > ANGLEMLMAX)
            angleML = ANGLEMLMAX;
        else if (angleML < -ANGLEMLMAX)
            angleML = - ANGLEMLMAX;
    }

    /**
    * @return si le canon est bloqu vers le vecteur orientation.
    */
    public boolean isBlockingML()
    {
        return blockingML;
    }
    
    /**
    * Inverse l'tat du bloquage du canon.
    */
    public void changeBlocking()
    {
        if ((PlayTime.getTime() - timeBlockingML) > 500) //on peut changer d'tat toute les 0.5 secondes
        {
            blockingML = !blockingML; // on chnage l'tat du blocage de la tourelle
            timeBlockingML = PlayTime.getTime(); // on enregistre qu'il y a eu changement d'tat
        }
    }
	

        
    /**
    * @return une liste contenant les 3 sommets du vaisseau.
    */
    public Point2D.Double[] getPeaks()
    {

        return peaks;   
    }
    	
    /**
	 * @return les coordonnes des 3 sommets du vaisseau dans un polygon
     */
	public Polygon getPolygon()
    {
        Point2D.Double[] tab = getPeaks();
        int[] pointx = {(int) tab[0].getX(), (int) tab[1].getX(), (int) tab[2].getX()};
        int[]  pointy = {(int) tab[0].getY(), (int) tab[1].getY(), (int) tab[2].getY()};
        return new Polygon (pointx, pointy, 3);     
    }
    
        
	/**
	 * 
	 * Rafraichit la position du vaisseau
     * @param pointer Les coordonnes du pointeur de la souris
	 */
	public void refresh(Point2D.Double pointer)
    {
        //on modifie le vecteur directeur si il y a une acceleration
        if (accel > 0 )
        {
            setDirect( Geom.normalize(Geom.add(Geom.normalize(getOrien()), Geom.normalize(getDirect(), (getSpeed() / 1000D) * Config.getRefresh()) )) );
        }
        setTranslate (Geom.normalize(getDirect(), (getSpeed() / 1000D) * Config.getRefresh() ));
        refreshPeaks();
        
        posML = Geom.add(getPos(), Geom.normalize(getOrien(), - BACK));
        /* rotation de la tourelle de tir */
        
        if (!blockingML && pointer != null) 
        {          
            // on tourne le canon vers le pointer
            rotateML(pointer);
        }
        else // on a bloqu la tourelle ou le pointeur n'est pas dfini
        {
            // on prend comme point de rfrence la pointe du vaisseau
            rotateML(peaks[0]);
        }
        // on diminue la temprature du canon 
        drecraseTempML(TEMPSLOOSERATE * Config.getRefresh());
    }
    
    /**
    * Mets  jours la position des coins du vaisseau
    */
    public void refreshPeaks()
    {
        Point2D.Double p0 = Geom.translate(getOrien(), getPos());
        // on tourne le vecteur orient de angle et on reduit sa longeur  gToTarget 
        Point2D.Double p1 = Geom.translate(getPos(), Geom.normalize(Geom.rotate(getOrien(), angle), gToTarget)); 
        Point2D.Double p2 = Geom.translate(getPos(), Geom.normalize(Geom.rotate(getOrien(), -angle), gToTarget)); 
        Point2D.Double[] tab = {p0, p1, p2};
        peaks = tab;
    }
    
    /**
    * Fonction permettant faire un tir de missile
    * Le vaisseau a un recule  chaque tir
    * @return un missile
    */
    public Missile fire()
    {
        // la vitesse du missile dpent de la vitesse du vaisseau
        //
        Point2D.Double posMissile = Geom.add(posML, Geom.normalize(getMLOrien(), sizeML + Missile.getSize()));
        Missile missile = new Missile(posMissile, getMLOrien(), 150 + getSpeed());
        
        /* 
            Effet de recul et de rotation du vaisseau 
            1 Faire un vecteur de recule de 1px dans le sens inverse au canon
            2 Reculer la position du canon (de magnanire fictive)
            3 La base du canon, le centre de gravit et le point du vaisseau devant tre aligne
                On calcule le vecteur entre le centre de gravit et le canon
                On modifie le vecteur orientation pour qu'il soit align sur celui si
            4 RefreshPosition s'occupe de replacer la position du canon
        */
        
        // on fait reculer le canon pour avoir un effet
        // on recule la base du canon du 1px
        Point2D.Double vBack = Geom.normalize(getMLOrien(), -1); 
        // on calcule la position temporaire du canon
        Point2D.Double posMLtemps = Geom.add(posML, vBack);
        
        // on aligne l'orientation du vaisseau sur la nouvelle position du canon
        setOrien(Geom.normalize(Geom.vector(getPos(), posMLtemps), - Geom.magnitude(getOrien())));
        
        setTranslate(Geom.normalize(vBack));
        
        // On ne change pas la position du canon, a se fera tout seul avec le refresh position
        
        // on donne le temps du dernier tir
        lastMLFire = getLifeTime();
        
        // on augmente la temprature 
        incraseTempML(Config.getTempMLIncreaseRate());
        
        return missile;
    }
    
    /**
    * @return le vecteur orientation du lance missile norme longeur du lance missile
    */
    public Point2D.Double getMLOrien()
    {
        return Geom.normalize(Geom.rotate(getOrien(), angleML), sizeML);
    }

    /**
    * Centre le vaisseau au milien de la zone de jeu
    */
    public void centerPosition()
    {
        setPos( Config.getPCenter() ) ;
    }
    
	/**
	 * @return l'acclration du vaisseau
	 */
	public int getAcceleration()
    {
        return accel;
    }
    
    /**
	* @return le pourcentage d'acclration (entre 0 t 1)
    */
    public double getPourcentAccel()
    {
        return (accel/ACCELMAX);
    }  
    
    /**
	* @return le pourcentage de vitesse (entre 0 t 1)
    */
    public double getPourcentSpeed()
    {
        return (getSpeed()/SPEEDMAX);
    }
    
	/**
	 * 
	 * @param newAcceleration La nouvelle valeur de l'acclration.
	 */
	public void setAcceleration(int newAcceleration)
    {
        accel = newAcceleration;
    }
    
    /**
    * Ajoute un en acceleration
    * Limit  ACCELMAX
    */
    public void increaseAcceleration()
    {
        if (accel < ACCELMAX)
        {
            accel++;
        }
    }    
    
    /**
    * Ajoute un en acceleration
    */
    public void decreaseAcceleration()
    {
        if (accel > 0)
        {
            accel--;
        }
    }
    
	/**
	 * Incrmente la vitesse du vaisseau de une unit
     * Vitesse limit  SPEEDMAX.
	 */
	public void increaseSpeed()
    {
        if (getSpeed() < SPEEDMAX) setSpeed(getSpeed() + 4) ;
    }
	
	/**
	 * dcrmente la vitesse du vaisseau de une unit
	 */
	public void decreaseSpeed()
    {
        if (getSpeed() > 0) setSpeed(getSpeed() - 2) ;
    } 
    
    /**
    * @return true si un point percute le vaisseau
    */
    public boolean isInterPoint(Point2D.Double p)
    {
        return getPolygon().contains(p);
    }     
    
    
    /**
    * @return true si un cercle percute le vaisseau
    */
    public boolean isInterCircle(Point2D.Double p, double r)
    {
        // On vrifie si le cercle est asss pret que pour avoir une possible collision
        if (Geom.distance(p, getPos()) <= (r + getRadius()))
        {
            // on calcule le point le plus proche du vaisseau
            Point2D.Double pCircleNearShip = Geom.add(getPos(), Geom.normalize(Geom.vector(getPos(), p), (Geom.distance(getPos(), p) - r)));
            return isInterPoint(pCircleNearShip) || (Geom.distance(p, peaks[0]) <= r) || (Geom.distance(p, peaks[1]) <= r) || (Geom.distance(p, peaks[2]) <= r);
            
        }
        else
            return false;
    }
    /**
    * Fait tourner le vaisseau vers la gauche.
    */
    public void turnLeft()
    {
        setOrien(Geom.rotate(getOrien(), -Config.getTurnFactor() ) );
    }
    
    /**
    * Fait tourne le vaisseau vers la droite.
    */
    public void turnRight()
    {
        setOrien(Geom.rotate(getOrien(), Config.getTurnFactor()) );
    }
    
    /**
    * @return un rayon pour approximer les collisions
    */
    public double getRadius()
    {
        return radius;
    }
    
    /**
    * @return le vecteur de translation pour deplacer les autres mobiles
    */
    public Point2D.Double getTranslate()
    {
        return vTranslate;
    }   
 
    /* On s'occupe de la temprature du canon */
    
    
    /**
    *   Diminue la temperature en fonction de t.
    *   @param t temprature perdue.
    */
    public void drecraseTempML(double t)
    {
        incraseTempML(-t);
    }    
    
    /**
    *   Augemente la temperature en fonction de t.
    *   @param t temprature gagne.
    */
    public void incraseTempML(double t)
    {
        if (tempML + t > MAXTEMPML) // on est en surchauffe
        {
            tempML = MAXTEMPML; // on mets la temperature au max;
            overheatingML = true; // on mets el canon en surchauffe.
        }
        else 
        {   
            if (tempML + t > 0)
                tempML +=  t;
            else
                tempML = 0;
            if (tempML <= ENDOVERHEATINGML)
            {
                // Si la tempratur passe sous la barre des ENDOVERHEATINGML on peut de nouveaux tirer.
                overheatingML = false; 
            }
        }
    }
    
    /**
    * @return La temprature du canon.
    */
    public double getTempML()
    {
        return tempML;
    }
    
    /**
    * Mets la tempratur du canon  0
    */
    public void setBonusTemp()
    {
        tempML = 0;
    }  
    
    /**
    * Mets la tempratur du canon  100%
    */
    public void setMalusTemp()
    {
        tempML = MAXTEMPML;
    }
    

    /**
    * @return si le canon est en surchauffe.
    */ 
    public boolean isOverheated()
    {
        return overheatingML;
    }
    
    /**
    * @return true si on peut lancer un nouveau missile.
    * Un missile peut tre tirer toutes les <code>getMissileTimeInterFire()</code> millisecondes.
    * Le canon ne doit pas tre en surchauffe
    */
    public boolean canFire()
    {
        // on retourne vrai si on a tir depuis X milliseconde et que le canon n'est pas en surchauffe
        return ((getLifeTime() - lastMLFire) >= Config.getMissileTimeInterFire()) &&  !overheatingML;
    }
    
    /**
    * Couleur :
    *   Temprature 100 - 75 : Rouge
    *   Temprature 75 - 50 : Organe
    *   Temprature 0 - 50 : Vert
    *
    * @return une couleur pour la jauge de temprature du lance missile.
    */
    public Color getColorDisplayTemp()
    {
        if (tempML > 75)
            return Color.red;
        else if (tempML > 50)
        {
            // Code RGB de l'orange obtenue via Photoshop
            return new Color(255, 30, 0);
        }
        else 
            return Color.green;
    }    
    
    /**
    * @return une couleur signifiant si on peut tirer ou non (vert oui, rouge non).
    */
    public Color getColorDisplayCanFire()
    {
        if (canFire())
            return Color.green;
        else 
            return Color.red;
    }
        
    /**
     * Mthode pour dessiner un vaisseau.
     */
    public void draw(Graphics g)
    {
        // Pour verifier la position du l'image > mettre le jeu en 900*720 ET midifier config.getFrame pour retourner 1
        // g.setColor(Color.yellow);
        // g.fillPolygon(getPolygon());

        /*Affichage du vaisseau */
        
        /* choix de l'image en fct du fait qu'on est invulnrable ou non */
        Image imageShip  = Engine.images.spaceShip;
        if (Main.getGame().getLife().isInvulnerable())
            imageShip  = Engine.images.spaceShipInv;
            
            
        Graphics2D g2 = (Graphics2D) g;
        /* En premier lieu l'image du vaisseau */
        // calcul du coin o mettre l'image
        // Les valeurs -141.5 et 49.82 px on tait obtenue via l'outils de mesure de Photoshop.
        Point2D.Double pCorner = Geom.translate(getPos(), Geom.normalize(Geom.rotate(getOrien(), -141.5), 49.82));
        AffineTransform at = new AffineTransform();
        at.translate(pCorner.getX() * Config.getFrameRate(), pCorner.getY() * Config.getFrameRate());
        at.rotate(Math.toRadians(Geom.angle(getOrien())));
        at.scale(Config.getFrameRate(), Config.getFrameRate());
        g2.drawImage(imageShip, at, null);    
        
        
        /* en second celle du lance missile */
        pCorner = Geom.translate(posML, Geom.normalize(Geom.rotate(getMLOrien(), -127.5), 11.34));
        at = new AffineTransform();
        at.translate(pCorner.getX() * Config.getFrameRate(), pCorner.getY() * Config.getFrameRate());
        // on tourne de l'orientation de v orient + de l'angle du canon
        at.rotate(Math.toRadians(Geom.angle(getOrien()) + angleML));
        at.scale(Config.getFrameRate(), Config.getFrameRate());
        g2.drawImage(Engine.images.missileLuncher, at, null);
        
        
        
        /* Affichage de la temprature du canon */
        
        // emplacement du coint superieur gauche de l'image de la jauge de temprature
        Point2D.Double pConerGauge = new Point2D.Double(50, 450); // coordonne obtenue via photoshop
        
        // On dssine un rectange representant la temprature du canon
        g.setColor(getColorDisplayTemp());
        double heigth = getTempML() * 1.8; // hauteur de la barre de temperature
        pCorner = new Point2D.Double(pConerGauge.getX() + 1, (pConerGauge.getY() + 1 + (180 - heigth)) ); 
        g.fillRect((int)(pCorner.getX() * Config.getFrameRate()), (int)(pCorner.getY() * Config.getFrameRate()),(int)( 64 * Config.getFrameRate()), (int)( heigth * Config.getFrameRate()));
        
        // On dcine un rectange signifiant si on peut tirer ou non
        g.setColor(getColorDisplayCanFire());
        pCorner = new Point2D.Double(pConerGauge.getX() + 1, pConerGauge.getY() + 7); 
        g.fillRect((int)(pCorner.getX() * Config.getFrameRate()), (int)(pCorner.getY() * Config.getFrameRate()),(int)( 16 * Config.getFrameRate()), (int)( 16 * Config.getFrameRate()));
        
        // Affichage de l'image de la jauge de temprature 
        at = new AffineTransform();
        at.translate(pConerGauge.getX() * Config.getFrameRate(), pConerGauge.getY() * Config.getFrameRate());
        at.scale(Config.getFrameRate(), Config.getFrameRate());
        g2.drawImage(Engine.images.missileLuncherGauge, at, null); 
        
        /* Affichage d'une jauge d'acclration et vitesse*/
        
        Point2D.Double pBarre = new Point2D.Double(50, 110);
        double width = 80;
        double height = 10;
        g.setColor(Color.blue);
        g.fillRect((int)(pBarre.getX() * Config.getFrameRate()), (int)(pBarre.getY() * Config.getFrameRate()), (int)( width * getPourcentAccel() * Config.getFrameRate()), (int)((height/2) * Config.getFrameRate()));
        
        g.setColor(Color.white);
        g.fillRect((int)(pBarre.getX() * Config.getFrameRate()), (int)((pBarre.getY() + 5) * Config.getFrameRate()), (int)( width * getPourcentSpeed() * Config.getFrameRate()), (int)((height/2) * Config.getFrameRate()));
        
        //contour de la barre

        g.drawRect((int)(pBarre.getX() * Config.getFrameRate()), (int)(pBarre.getY() * Config.getFrameRate()), (int)( width * Config.getFrameRate()), (int)(height * Config.getFrameRate()));
    }  
}