package be.ac.umons.ProjetInfo.BAC1.CharlierDucruet.geom;

import java.awt.geom.Point2D;
import java.util.Random;
/**
* La classe Geom  pour but de fournir les fonctions gometrique.
*/
public class Geom
{
   /**
    * Utilise comme indice d'erreur pour les comparaisons.<br/>
    * La valeur de retour est <code>0.00000000001</code> (un double).
    */
    public static final double EPSILON = 0.0000001;
    
   /**
    *Cre un vecteur allant de <code>p1</code>  <code>p2</code>.
    *
    *@param p1 Point de dpart.
    *@param p2 Point d'arrive.
    *@return Un vecteur allant de <code>p1</code>  <code>p2</code>.
    */
    public static Point2D.Double vector(Point2D.Double p1, Point2D.Double p2)
	{
        double x1 = p1.getX();
        double y1 = p1.getY();
        double x2 = p2.getX();
        double y2 = p2.getY();
        return new Point2D.Double ((x2-x1), (y2-y1));
    }
    
   /**
    * Additionne deux vecteurs.
    * @param v1 Un vecteur.
    * @param v2 Un vecteur.
    * @return Un vecteur.
    */
    public static Point2D.Double add(Point2D.Double v1, Point2D.Double v2)
	{
        double x1 = v1.getX();
        double y1 = v1.getY();
        double x2 = v2.getX();
        double y2 = v2.getY();
        return new Point2D.Double ((x1+x2), (y1+y2));
    }
        
   /**
    * Translation d'un point par un vecteur.
    * @param p Un point.
    * @param v Un vecteur.
    * @return Un nouveau point rsultant de la translation de <code>p</code> par <code>v</code>.
    */ 
	public static Point2D.Double translate(Point2D.Double p,Point2D.Double v)
	{
		return add(p, v);
	}
    
   /**
    * Calcule la distance entre les 2 points.
    * @param p1 Un point.
    * @param p2 Un point.
    * @return Un double.
    */    
    public static double distance(Point2D.Double p1, Point2D.Double p2) 
	{        
        double x, y;
        // on prend la plus grande difference entre les coordonnes (pour que a fonctionne aussi dans les  ngatif)
        x = Math.max((p1.getX() - p2.getX()), (p2.getX() - p1.getX()));
        y = Math.max((p1.getY() - p2.getY()), (p2.getY() - p1.getY()));
        return Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2));
    }
    
   /**
    * Calcule la norme d'un vecteur.
    * @param v Un vecteur.
    * @return Un double.
    */  
    public static double magnitude(Point2D.Double v)
	{
        return distance(new Point2D.Double(0,0), v);
    }
    
   /**
    * Cre un vecteur  partir du vecteur d'entre, mais avec une norme diffrente.
    * @param v Un vecteur.
    * @param n Une norme.
    * @return Un nouveau vecteur.
    */  
    public static Point2D.Double normalize(Point2D.Double v, double n)
    {
		double norme = magnitude(v);
        if (Math.abs(norme) < EPSILON){return new Point2D.Double();}
		double coefficient = n/norme;
		double x = v.getX();
		double y = v.getY();
		return new Point2D.Double(x * coefficient, y * coefficient); 
	}
    
    
   /**
    * Cre un vecteur  partir du vecteur entr, mais avec une norme de 1.
    * @param v Un vecteur.
    * @return Un nouveau vecteur.
    */  
	public static Point2D.Double normalize(Point2D.Double v) 
	{
		return normalize(v, 1.0);
    }
    
   /**
    * Donne l'angle (en degrs) entre deux vecteurs (entre -180 et 180) de v1 vers v2 (sans anti-horlog).
    * @param v1 Un vecteur.
    * @param v2 Un vecteur.
    * @return Langle (en degrs) entre <code>v1</code> et <code>v2</code>. 
    */  
    public static double angle(Point2D.Double v1, Point2D.Double v2)
    {
		double degrees_v1 = angle(v1);
		double degrees_v2 = angle(v2);
        // On obtient une valeur entre 0 et 360 qui est l'angle en sens horlogique
        double angle = Math.max(degrees_v1, degrees_v2) - Math.min(degrees_v1, degrees_v2);
        
        if (degrees_v1 > degrees_v2) 
        {
            if (angle > 180)
            {
                return (360 - angle);
            }
            return - angle;
        }
        else
        {
            if (angle > 180)
            {
                return -(360 - angle);
            }
            return angle;
        }
    }
    
   /**
    * Donne langle (en degrs) entre un vecteur et le vecteur origine (sens anti-horlog).
    * @param v Un vecteur.
    * @return Langle (en degrs) entre <code>v</code> et le vecteur origine (1, 0) (sens anti-horlog). 
    */  
    public static double angle(Point2D.Double v)
    {
		double x = v.getX();
		double y = v.getY();
        if (x == 0 && y >= 0)
            return 90;
        if (x == 0 && y < 0)
            return 270;
        double angle = Math.toDegrees(Math.atan(y / x));
		if (x >= 0 && y >= 0) 
		{
			// on est dans le 1er quadrant
			return angle;
		}		
		else if ((x <= 0 && y >= 0) || (x <= 0 && y <= 0))
		{
			//on est dans le 2me ou le 3me quadrant
			return 180 + angle;
		}
		else 
		{
			// on est dans le 4me quadrant
			return 360 + angle;
		}
    }
    
   /**
    * Gnre le vecteur rsultant de la rotation de <code>v</code> par <code>angle</code> (sens anti-horlog).
    * @param v Un vecteur.
    * @param _angle Un angle en degrs.
    * @return Le vecteur rsultant de la rotation de <code>v</code> par <code>_angle</code> (sens anti-horlog).
    */  
	public static Point2D.Double rotate(Point2D.Double v, double _angle)
	{

		double angle = Math.toRadians(_angle);
        double cos = Math.cos(angle);
        double sin = Math.sin(angle);
        double x = v.getX() * cos - v.getY() * sin;
        double y = v.getX() * sin + v.getY() * cos;
        return new Point2D.Double(x,y);
	}
    
   /**
    * Gnere un nouveau point rsultant de la rotation du vecteur <code>vecteur</code> par l'angle <code>angle</code> (sens anti-horlog> en considrant que le point est au bout du vecteur.
    * @param v Un vecteur.
    * @param p Un point.
    * @param angle Un angle en degrs.
    * @return Un nouveau point rsultant de la rotation du vecteur <code>vecteur</code> par l'angle <code>angle</code> (sens anti-horlog) en considrant que le point est au bout du vecteur.
    */  	
	public static Point2D.Double rotate(Point2D.Double v, Point2D.Double p, double angle)
	{
		Point2D.Double vecteurRotate =  rotate(v, angle); //vecteur aprs rotation
		Point2D.Double vecteurDeTranslation = vector(v, vecteurRotate); //vecteur issu des 2 sommets des vecteurs
		return translate(p, vecteurDeTranslation);
	}

    
   /**
    * Gnre un point alatoirement dans un "cadre" dlimit par <code>_x</code> et <code>_y</code>.
    * @param _x Une valeur max pour l'axe des x.
    * @param _y Une valeur max pour l'axe des y.
    * @return Un point gnr alatoirement dans un intervale X de [0,<code>_x</code>] et Y de [0,<code>_y</code>].
    */
    public static Point2D.Double pointRandom(double _x, double _y)
	{
        double x= Math.random() * _x;
        double y= Math.random() * _y;
        return new Point2D.Double(x, y);
    }
    
   /**
    * Gnre un vecteur de direction alatoire.
    * @param norme Une norme pour le vecteur.
    * @return Un vecteur de direction alatoire et de norme <code>norme</code>.
    */
	public static Point2D.Double vectorRandom(double norme)
	{
        double angle = Math.random() * 360;
		return normalize(Geom.rotate(new Point2D.Double(1, 1), angle), norme);
	}   
    
    /**
    * Gnre un vecteur de direction alatoire.
    * @return Un vecteur de direction alatoire et de norme 1.
    */
	public static Point2D.Double vectorRandom()
	{
        double angle = Math.random() * 360;
		return Geom.rotate(new Point2D.Double(1, 1), angle);
	}
    
   /**
    * Gnre un point sur le bord de l'cran.<br />
    * En fonction de <code>x</code> et de <code>y</code>.
    * @param x La largeur de la fentre.
    * @param y La hauteur de la fentre.
    * @return Un point.
    */
    public static Point2D.Double pointRandomBorder(int x, int y)
    {
        
        Random rand = new Random();
        double i = rand.nextInt(4); //va tre utilis pour choisir "de quel cot" sera plac le point gnr.
		if (i == 1)
		{ // on se place en x = 0
			return new Point2D.Double(0, rand.nextInt(y));
		}
		else if (i == 2)
		{ // on se place en y = 0
			return new Point2D.Double(rand.nextInt(x), 0);
		}		
		else if (i == 3)
		{ // on se place en y = y
			return new Point2D.Double(rand.nextInt(x), y);
		}		
		else
		{ // on se place en x = x
			return new Point2D.Double(x, rand.nextInt(y));
		}
    }
    
   /**
    * Evalue si 2 cercles on des points d'intersections.
    * @param c0 un cercle.
    * @param r0 le rayon du cercle <code>c0</code>.
    * @param c1 un cercle.
    * @param r1 la rayon du cercle <code>c1</code>.
    * @return <code>true</code> si les 2 cercles ont une ou 2 intersection(s) ou si un cercle est contenu dans un autre, <code>false</code> sinon.
    */
    public static boolean isInterTwoCircles(Point2D.Double c0, double r0, Point2D.Double c1, double r1)
    {
        double distanceCentre = distance(c0, c1);
        if (distanceCentre > Math.max(r0, r1)) 
        {
            double sommeRayon = r0 + r1;
            // si |c0, c1| <= r0 + r1 <= 2|c0, c1| alors les 2 cercles s'intercepte
            return (distanceCentre <= sommeRayon && sommeRayon <= 2 * distanceCentre);
        }
        else
        {
            return true; // un des cercles est contenu dans l'autre, ou il y a au moins une intersection
            // double sommeRayon = Math.abs(r0 - r1);
            // return (sommeRayon <= distanceCentre);
        }
    }
    
    /**
    * Evalue si un point est "dans" un cercle (pour les collision).
    * @param c le cerntre du cercle.
    * @param r le rayon du cercle
    * @param p le point.
    * @return true si le point est dans le cercle, false sinon.
    */
    public static boolean isInterCiclePoint(Point2D.Double c, double r, Point2D.Double p)
    {
        return distance(c, p) <= r;
    }
   
    
   /**
    * Inverse le sens du vecteur
    * @param v un vecteur
    */
    public static Point2D.Double inverseVector(Point2D.Double v)
    {
        return new Point2D.Double (-v.getX(), -v.getY());
    }    
	
   /**
    * Permet de faire rapararaitre le mobile au bon endroit
    */
    public static Point2D.Double frameModulo(Point2D.Double p, Point2D.Double vecteur, double frameX, double frameY)
    {
        double pX = p.getX();
        double pY = p.getY();
        Point2D.Double v = normalize(vecteur);
        while (pX < 0 || pX > frameX || pY < 0 || pY > frameY)
        {
            if (pX < 0) 
            {
                pX = frameX + pX;
                pY = pY + (v.getY() * 50);
            }
            if (pX > frameX) 
            {
                pX = pX % frameX;
                pY = pY + (v.getY() * 50);
            }
            if (pY < 0) 
            {
                pY = frameY + pY;
                pX = pX - (v.getX() * 50);
            }
            if (pY > frameY)        
            {
                pY = pY % frameY;
                pX = pX - (v.getX() * 50);
            }   
        }
        return new Point2D.Double (pX, pY);
    }     
    
    public static Point2D.Double frameModulo(Point2D.Double p, double frameX, double frameY)
    {
        return new Point2D.Double((p.getX() + frameX) % frameX, (p.getY() + frameY) % frameY);
    }  
    
    /**
    *   Permet de savoir si un point se situe  une distance d du bord d'un cadre x*y
    * @param p un point
    * @param d une distance
    * @param _x la largeur du cadre
    * @param _y la hauteur du cadre
    */
    public static boolean isCloseBorder(Point2D.Double p, double d, int _x, int _y)
    {
        return ( (p.getX() <= d) || (p.getX() >= (_x - d)) || (p.getY() <= d) || (p.getY() >= (_y - d)) );
    }    
    
    /**
    *   @return la position du point clone 
    *   "clone" car dcal pour simuler le dbordement par apport au cadre.
    * @param p un point
    * @param d une distance
    * @param _x la largeur du cadre
    * @param _y la hauteur du cadre
    */
    public static Point2D.Double pCloseBorder(Point2D.Double p, double d, int _x, int _y)
    {
        double x = p.getX();
        double y = p.getY();
        if (p.getX() <= d)
        {
            x = _x + p.getX();
        }
        else if (p.getX() >= (_x - d))
        {
            x = p.getX() - _x ;
        }
        if (p.getY() <= d)
        {
            y = _y + p.getY();
        }
        else if (p.getY() >= (_y - d))
        {
            y = p.getY() - _y ;
        }
        return new Point2D.Double(x, y);   
    }
    
    /**
    *   Permet de savoir si un point se situe  une distance d du bord d'un cadre x*y ou dans celui si
    * @param p un point
    * @param d une distance
    * @param _x la largeur du cadre
    * @param _y la hauteur du cadre
    */
    public static boolean isCloseOrInBorder(Point2D.Double p, double d, int _x, int _y)
    {
        d = Math.ceil(d);
        return ( (p.getX() >= - d) && (p.getX() <= (_x + d)) && (p.getY() >= -d) && (p.getY() <= (_y + d)) );
    }     
    
    /**
    *   Permet de savoir si un point se situe dans un cadre x*y
    * @param p un point
    * @param _x la largeur du cadre
    * @param _y la hauteur du cadre
    */
    public static boolean isCloseOrInBorder(Point2D.Double p, int _x, int _y)
    {
        return isCloseOrInBorder(p, 0D, _x, _y);
    }    
    
    /**
    *   @param c le centre du cercle
    *   @param r le rayon du cercle
    *   @param p le point o l'on veut appliquer le modulo.
    *   @return un point rsultant de la sysmtrie centrale du point par apport au centre du cercle si il est  une distance rayon du cercle.
    */
    public static Point2D.Double circleModulo(Point2D.Double p, Point2D.Double c, double r)
    {
        if (distance(c, p) >= r)
        {
            // on deplace le point p avec une symetrie de distance r -0.5 (pour ne pas avoir de modulo infini).
            return Geom.add(c, Geom.normalize(Geom.vector(p, c), (r - 0.5D) ));
        }
        else
            return p;
    }
    
    /**
    * @param c le centre du cercle
    * @param r le rauon du cercle.
    * @return un point allatoirement plac sur le bord d'un cercle.
    */
    public static Point2D.Double randomPointEdgeCircle(Point2D.Double c, double r)
    {
        // on cr un vecteur de longeur r pour ensuite le tourner d'un angle allatoire et l'additionner au centre du cercle
        Point2D.Double v = new Point2D.Double(r, 0);
        Random rand = new Random();
        v = rotate(v, rand.nextInt(360));
        return add(c, v);
    }
    

}